package com.takensoft.taken_bi_manager.common.connection.db.util;

import java.sql.SQLException;

import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.SqlSessionTemplate;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;

import com.takensoft.taken_bi_manager.common.connection.db.vo.ConnectionDB;
import com.takensoft.taken_bi_manager.common.vo.SystemCode.DatabaseType;
import com.zaxxer.hikari.HikariDataSource;

/**
 * @author 김성원
 * @since 2024.01.02
 * 
 * 동적 데이터 베이스 연계 관리 클래스 입니다.
 */
public class DBConnectionConfig {
	
	//프로젝트 내부 Class Path 정보
	// 한개 logger.debug(pathMatchingResourcePatternResolver.getResource("절대경로 혹은 classpath:이하경로").getFile().getAbsolutePath()); 
	// 여러개 Resource[] resource = pathMatchingResourcePatternResolver.getResources("절대경로 혹은 classpath:이하경로");
	private final static PathMatchingResourcePatternResolver resourceResolver = new PathMatchingResourcePatternResolver();
	
	//동적 Mybatis 연계 - mybatis 설정 정보 파일 경로
	private final static String DYNAMIC_MYBATIS_CONFIG = "com/takensoft/taken_bi_manager/common/connection/db/mybatis/config/config.xml";
	
	//동적 Mybatis 연계 - mybatis Mapper 문서 파일 경로 
	private final static String DYNAMIC_MYBATIS_MAPPERS = "com/takensoft/taken_bi_manager/common/connection/db/mybatis/sql/*-SQL.xml";
	
	private HikariDataSource dataSource = null;
	
	private SqlSessionFactoryBean sqlSession = null;
	
	private SqlSessionTemplate template = null;
	
	private DataSourceTransactionManager dataSourceTransaction = null;
	
	public HikariDataSource getDataSource() {
		return dataSource;
	}

	public void setDataSource(HikariDataSource dataSource) {
		this.dataSource = dataSource;
	}

	public SqlSessionFactoryBean getSqlSession() {
		return sqlSession;
	}

	public void setSqlSession(SqlSessionFactoryBean sqlSession) {
		this.sqlSession = sqlSession;
	}

	public SqlSessionTemplate getTemplate() {
		return template;
	}

	public void setTemplate(SqlSessionTemplate template) {
		this.template = template;
	}


	/**
	 * @author 김성원
	 * @since 2024.01.08
	 * 
	 * 동적 DB + Mybatis 객체 생성 -> Mapper에 접근하기위한 SQL 템플릿 반환
	 */
	public SqlSessionTemplate mybatisSqlTemplateCreate (ConnectionDB db) {
		dataSource = dataSourceCreate(db);
		sqlSession = sqlSessionCreate(dataSource);
		template = sqlSessionTemplateCreate(sqlSession);		
		dataSourceTransaction = getTransaction(dataSource);
		
		return template;
	}
	
	/**
	 * @author 김성원
	 * @since 2024.01.08
	 * 
	 * 동적 DataSource 생성
	 */
	public HikariDataSource dataSourceCreate (ConnectionDB db) {
		dataSource = null;
		try {
			dataSource = new HikariDataSource();
			DatabaseType dataBaseType = db.getDatabaseType();
			dataSource.setDriverClassName(dataBaseType.getDriverClassName());
			dataSource.setJdbcUrl(dataBaseType.getUrl(db));
			
			dataSource.setUsername(db.getUserId());
			dataSource.setPassword(db.getUserPassword());
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return dataSource;
	}
	
	/**
	  * @author 김성원
	 * @since 2024.01.08
	 * 
	 * 동적으로 생성한 DataSource정보와 Mybatis 설정 정보 연계 생성
	 */
	private SqlSessionFactoryBean sqlSessionCreate (HikariDataSource dataSource) {
		try {
			sqlSession = new SqlSessionFactoryBean();
			sqlSession.setDataSource(dataSource);
			sqlSession.setConfigLocation(resourceResolver.getResource(DYNAMIC_MYBATIS_CONFIG));
			sqlSession.setMapperLocations(resourceResolver.getResources(DYNAMIC_MYBATIS_MAPPERS));
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return sqlSession;
	}
	
	/**
	 * @author 김성원
	 * @since 2024.01.08
	 * 
	 * 동적 Mybatis Mapper에 접근하기위한 SQL 템플릿 생성
	 */
	private SqlSessionTemplate sqlSessionTemplateCreate (SqlSessionFactoryBean sqlSession) {
		try {
			template = new SqlSessionTemplate(sqlSession.getObject());
			template.clearCache();
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		return template;
	}
	
	
	/**
	 * @author 김성원
	 * @since 2024.01.08
	 * 
	 * 동적 Mybatis Mapper에 접근하기위한 SQL 템플릿 생성
	 */
	private DataSourceTransactionManager getTransaction (HikariDataSource dataSource) {
		
		this.dataSourceTransaction = new DataSourceTransactionManager(dataSource);
	
		return dataSourceTransaction;
	}	
	
	
	
	/**
	 * @author 김성원
	 * @since 2024.01.08
	 * 
	 * 커넥션 닫기
	 */
	public void close() throws SQLException {
		try {
			if (dataSource != null) {
				dataSource.evictConnection(dataSource.getConnection());
				if (dataSource.getConnection() != null) {
					dataSource.getConnection().close();
				}
				dataSource.close();
			}
		}catch(Exception e) {
			if (dataSource != null) {
				dataSource.close();
			}
		}
		
	}
	

}
