package com.takensoft.taken_bi_manager.common.connection.db.util;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.transaction.annotation.Transactional;

import com.takensoft.taken_bi_manager.common.connection.db.dao.DatabaseDAO;
import com.takensoft.taken_bi_manager.common.connection.db.dao.MariadbDAO;
import com.takensoft.taken_bi_manager.common.connection.db.dao.PostgresqlDAO;
import com.takensoft.taken_bi_manager.common.connection.db.vo.ConnectionDB;
import com.takensoft.taken_bi_manager.common.connection.db.vo.TableBasicInfo;
import com.takensoft.taken_bi_manager.common.util.CommonUtil;
import com.takensoft.taken_bi_manager.common.util.StringUtil;
import com.takensoft.taken_bi_manager.common.vo.CheckMessage;
import com.takensoft.taken_bi_manager.common.vo.SystemCode.DatabaseType;
import com.takensoft.taken_bi_manager.data.vo.ColumnData;
import com.takensoft.taken_bi_manager.data.vo.DataTable;
import com.takensoft.taken_bi_manager.data.vo.Dataset;



/**
 * @author 최정우
 * @since 2019.11.12
 * 
 * 데이터 수집 - DB 데이터 연계 관련 Util 입니다. 
 */
@Transactional
public class DBConnectionUtil {
	
	private Logger logger = LoggerFactory.getLogger(DBConnectionUtil.class);

	/**
	 * @author 김성원
	 * @since 2024.01.04
	 * 
	 * 데이터베이스 접속 모듈 DAO 객체 생성 메서드입니다.
	 */
	public DatabaseDAO createDatabaseDAO (ConnectionDB db) throws Exception {
		
		DatabaseDAO databaseDao= null;
		
		if (db.getDatabaseType() == DatabaseType.MYSQL) {
	//		databaseDao =  new MysqlDAO(db);
		} else if (db.getDatabaseType() == DatabaseType.MARIADB) {
			databaseDao =  new MariadbDAO(db);
		} else if (db.getDatabaseType() == DatabaseType.ORACLE) {
		//	databaseDao =  new OracleDAO(db);
		} else if (db.getDatabaseType() == DatabaseType.MSSQL) {
		//	databaseDao =  new MssqlDAO(db);
		} else if (db.getDatabaseType() == DatabaseType.TIBERO) {
		// databaseDao =  new TeberoDAO(db);
		}else if (db.getDatabaseType() == DatabaseType.POSTGRESQL) {
		 	return new PostgresqlDAO(db);
		} else {
			throw new Exception();
		}
		return databaseDao;
	}
	
	/**
	 * @author 김성원
	 * @since 2024.01.04
	 * 
	 * DB 접속 테스트 입니다.
	 */
	public CheckMessage dbConnectionCheck (ConnectionDB dbCollection) throws Exception {
		if(StringUtil.isEmpty(dbCollection.getConectIp()) == true) {
			return new CheckMessage(false, "접속 IP를 입력해주세요");
		} else if(StringUtil.isEmpty(dbCollection.getConectPort()) == true) {
			return new CheckMessage(false, "접속 Port를 입력해주세요");
		} else if(CommonUtil.isInt(dbCollection.getConectPort()) == false) {
			return new CheckMessage(false, "접속 Port는 숫자로만 입력해주세요");
		} else if(StringUtil.isEmpty(dbCollection.getDatabaseNm()) == true) {
			return new CheckMessage(false, "DB명을 입력해주세요");
		} else if(StringUtil.isEmpty(dbCollection.getConectIp()) == true) {
			return new CheckMessage(false, "접속 ID를 입력해주세요.");
		} else if(StringUtil.isEmpty(dbCollection.getUserPassword()) == true) {
			return new CheckMessage(false, "접속 PW를 입력해주세요.");
		} else {
			/*if (CommonUtil.pingCheck(dbCollection.getIp()) == false) {
				return new CheckMessage(false, "접속 IP(" + dbCollection.getIp() + ")와 연결할 수 없습니다");
			}*/
			if (CommonUtil.linkCheck(dbCollection.getConectIp(), Integer.parseInt(dbCollection.getConectPort())) == false) {
				return new CheckMessage(false, "접속 IP(" + dbCollection.getConectIp() + ")의 Port(" + dbCollection.getConectPort() + ")와 연결할 수 없습니다");
			} 
			Connection connection = null;
			DBConnectionConfig dbUtil = new DBConnectionConfig();
			try {
				dbUtil.dataSourceCreate(dbCollection);
				connection = dbUtil.getDataSource().getConnection();
				return new CheckMessage(true, "[" + dbCollection.getDatabaseType().getValue() + "] " + dbCollection.getDatabaseNm() + " 접속 성공");
			} catch (SQLException e) {
				System.out.println(e.toString());
				return new CheckMessage(false, "접속 에러 : " + e.toString());
			} catch (Exception e) {
				System.out.println(e.toString());
				return new CheckMessage(false, "접속 에러 : " + e.toString());
			} finally {
				if (connection != null) {
					try {
						connection.close();
					} catch (Exception e) {
						System.out.println(e.toString());
						return new CheckMessage(false, "접속 에러 : " + e.toString());
					}
				} if (dbUtil != null) {
					dbUtil.close();
				}
			}
		}
	}
	
	/**
	 * @author 김성원
	 * @since 2024.01.04
	 * 
	 * DB 접속 - 테이블 목록 조회 메소드 입니다.
	 */
	public List<Dataset> getDBConnectionTableList (ConnectionDB dbCollection) throws Exception {
		CheckMessage message = dbConnectionCheck(dbCollection);
		
		if (message.isSuccess() == false) {
			logger.info("getDbConnectionTableList DB연결실패 : " + message.getMessage());
			return new ArrayList<Dataset>();
		}
		
		List<Dataset> dbConnectionTableList = new ArrayList<Dataset>();
		
		//DAO 인스턴스 생성
		DatabaseDAO databaseDAO = createDatabaseDAO(dbCollection);
		
		try {
			//테이블 목록 조회
			 dbConnectionTableList = databaseDAO.getDBConnectionTableList(dbCollection);
		}finally {
			//DB연결 종료
			databaseDAO.dbConnectionClose();	
		}
			
		
		
		return dbConnectionTableList;
	}
	
	/**
	 * @author 김성원
	 * @since 2024.01.04
	 * 
	 * 커넥션된 데이터베이스의 테이블의 컬럼 정보 조회 (DB 기준 데이터 타입, 데이터 크기 등)
	 */
	public List<ColumnData> getDBConnectionTableColumnList(ConnectionDB dbCollection, Dataset dataset) throws Exception {
		CheckMessage message = dbConnectionCheck(dbCollection);
		
		if (message.isSuccess() == false) {
			logger.info("getDBConnectionTableColumnList DB연결실패 : " + message.getMessage());
			return new ArrayList<ColumnData>();
		}
		
		List<ColumnData> tableColumnList = new ArrayList<ColumnData>();
		
		//DAO 인스턴스 생성
		DatabaseDAO databaseDAO = createDatabaseDAO(dbCollection);
		
		try {
			//테이블 컬럼 목록 조회
			tableColumnList = databaseDAO.getDBConnectionTableColumnList(dataset);
		}finally {
			//DB연결 종료
			databaseDAO.dbConnectionClose();	
		}
		
		return tableColumnList;
	}
	
	/**
	 * @author 최정우
	 * @since 2020.01.28
	 * 
	 * 커넥션된 데이터베이스의 테이블의 데이터에 대한 데이터 테이블 조회 메서드입니다.
	 */
	public void setDataTableByDBConnectionTable (ConnectionDB dbCollection, DataTable dataTable) throws Exception {
		CheckMessage message = dbConnectionCheck(dbCollection);
		
		if (message.isSuccess() == false) {
			dataTable.setCheckMessage(message);
			logger.info("setDataTableByDBConnectionTable DB연결실패 : " + message.getMessage());
			return;
		}
		
		//DAO 인스턴스 생성
		DatabaseDAO databaseDAO = createDatabaseDAO(dbCollection);
		
		int tableDataTotalRows = 0;
		
		List<LinkedHashMap<String, Object>> tableDataMapList  = new ArrayList<LinkedHashMap<String, Object>>();
		
		try {
			
			//테이블 기본 Select 정보(데이터베이스명, 테이블명) 입력
			Dataset dataset = new Dataset();
			dataset.setDatabaseNm(dbCollection.getDatabaseNm());
			dataset.setTableNm(dataTable.getTableNm());		
			
			//테이블 컬럼 목록 조회
			dataTable.setColumnDatas(databaseDAO.getDBConnectionTableColumnList(dataset));//테이블 컬럼 정보 set
			dataTable.setColumnDatas( DataTypeUtil.convertDataTypeDbtoJavaList(dataTable.getColumnDatas(),dbCollection.getDatabaseType()));
			
			//테이블 데이터 조회
			if (dbCollection.getDatabaseType() == DatabaseType.ORACLE) {
				dataTable.setDatabaseNm(dbCollection.getUserId());
			}
			
			tableDataMapList = databaseDAO.getDBConnectionTableDataList(dataTable);
			dataTable.setRowData(CommonUtil.rowDataMapToList(tableDataMapList));//테이블 데이터 set
			
			//테이블 데이터 총 개수 조회
			tableDataTotalRows = databaseDAO.getDBConnectionTableDataTotalRows(dataTable);
			dataTable.setTotalRows(tableDataTotalRows);//테이블 데이터 총 개수 set
			
			//SELECT 쿼리 생성
			dataTable.setQuery();
			
		}catch(Exception e) {
			
			logger.error(e.toString());
			
		}finally {
			//DB연결 종료
			databaseDAO.dbConnectionClose();
		}
				
		//성공 메세지 입력
		dataTable.getCheckMessage().setSuccess(true);
		dataTable.getCheckMessage().setMessage("(총 조회된 행수 " + tableDataTotalRows + "개 / 현재 조회된 데이터 수 " + tableDataMapList.size() + "개)");
	}
	
	/**
	 * @author 김성원
	 * @since 2024.01.04
	 * 
	 * 커스텀 쿼리를 통한 dataTable setting (페이징사용 유무에 따라 페이징 처리 - {rowDatas, columnDatas, query(이미 존재함)})
	 */
	public void setDataTableByDBConnectionCustomQuery (ConnectionDB dbCollection, DataTable dataTable) throws Exception {
		CheckMessage message = dbConnectionCheck(dbCollection);
		
		if (message.isSuccess() == false) {
			dataTable.setCheckMessage(message);
			logger.info("setDataTableByDBConnectionCustomQuery DB연결실패 : " + message.getMessage());
			return;
		}
		
		int tableDataTotalRows = 0;
		
		List<LinkedHashMap<String, Object>> tableDataMapList  = new ArrayList<LinkedHashMap<String, Object>>();
		
		
		//DAO 인스턴스 생성
		DatabaseDAO databaseDAO = createDatabaseDAO(dbCollection);
		
		try {
			//테이블 데이터 조회
			tableDataMapList = databaseDAO.getDBConnectionCustomQueryDataList(dataTable);
			
			//DataTable에 조회된 데이터에서 첫번째 행 값의 컬럼 분리 후, columnDatas에 세팅
			//DataTable에 rowData 세팅
			if (tableDataMapList != null && tableDataMapList.size() > 0) {				
				
				//테이블 데이터 set
				dataTable.setRowData(CommonUtil.rowDataMapToList(tableDataMapList));				
				
				//컬럼명 set
				List<ColumnData> columnDatas = new ArrayList<ColumnData>();
				LinkedHashMap<String, Object> map = tableDataMapList.get(0);
				
				for(String key : map.keySet()){					
					ColumnData column = new ColumnData();
					column.setColumnNm(key);
					column.setDisplyColumnNm(key);
					column.setOrginlColumnNm(key);				
					// 데이터 타입 정의 기능 
					column.dataTyDefine(map.get(key));					
					columnDatas.add(column);	
				}
				
				dataTable.setColumnDatas(columnDatas);				
			}
			
			//테이블 데이터 총 개수 조회
			tableDataTotalRows = databaseDAO.getDBConnectionCustomQueryDataTotalRows(dataTable);
			dataTable.setTotalRows(tableDataTotalRows);//테이블 데이터 총 개수 set
		}catch(Exception e) {
			logger.error(e.toString());			
		}finally {
			//DB연결 종료
			databaseDAO.dbConnectionClose();
		}
		//DB연결 종료
		databaseDAO.dbConnectionClose();
		
		//성공 메세지 입력
		dataTable.getCheckMessage().setSuccess(true);
		dataTable.getCheckMessage().setMessage("(총 조회된 행수 " + tableDataTotalRows + "개 / 현재 조회된 데이터 수 " + tableDataMapList.size() + "개)");
	}
	
	
	/**
	 * @author 김성원
	 * @since 2024.01.04
	 * 
	 * 조회내용이 없는 실행형 커스텀 쿼리(update, delete 등)
	 */
	public int executeCustomQuery(ConnectionDB dbCollection, DataTable dataTable) throws Exception {
		
		DatabaseDAO databaseDAO = createDatabaseDAO(dbCollection);
		
		int result = 0;
		try {
			result = databaseDAO.executeCustomQuery(dataTable);
		}catch(Exception e) {
			
		}finally{
			databaseDAO.dbConnectionClose();	
		}
		
		return result;
	}
	
	
	/**
	 * @author 김성원
	 * @since 2024.01.04
	 * 
	 * 커스텀 쿼리 지정된 PK로 테이블 생성이 가능한지 확인
	 */
	public void setCustomQueryPrimaryCreatePossibleCheck (ConnectionDB dbCollection, DataTable dataTable) throws Exception {
		
		if (dataTable.getColumnDatas() == null || dataTable.getColumnDatas().size() == 0) {
			dataTable.getCheckMessage().setSuccess(false);
			dataTable.getCheckMessage().setMessage("PK생성 가능 유무 검사 - 테이블을 생성할 컬럼이 없습니다.");
			return;
		}
		
		String primaryColumns = "";
		List<ColumnData> primaryColumnDatas = new ArrayList<ColumnData>();
		
		for (int i = 0; i < dataTable.getColumnDatas().size(); i++) {
			if (dataTable.getColumnDatas().get(i).isPkAt() == true) {
				primaryColumns += ("[" + dataTable.getColumnDatas().get(i).getColumnNm() + "]");
				primaryColumnDatas.add(dataTable.getColumnDatas().get(i));
			}
		}
		
		if (primaryColumnDatas.size() == 0) {
			dataTable.getCheckMessage().setSuccess(true);
			dataTable.getCheckMessage().setMessage("PK생성 가능 유무 검사 - PK로 지정한 컬럼이 없습니다.(검사통과)");
			return;
		}
		
		//DB Connecting
		CheckMessage message = dbConnectionCheck(dbCollection);
		if (message.isSuccess() == false) {
			dataTable.setCheckMessage(message);
			logger.info("setDataTableByDBConnectionCustomQuery DB연결실패 : " + message.getMessage());
			return;
		}
		
		//DAO 인스턴스 생성
		DatabaseDAO databaseDAO = createDatabaseDAO(dbCollection);
		
		DataTable param = new DataTable();
		param.setColumnDatas(primaryColumnDatas);
		param.setQuery(dataTable.getQuery());
		
		//커스텀 쿼리 지정된 PK로 테이블 생성이 가능한지 확인
		int totalOverlapCount = databaseDAO.getCustomQueryPrimaryOverlapCount(param);
		
		//DB연결 종료
		databaseDAO.dbConnectionClose();
		
		//결과 메세지 입력
		if (totalOverlapCount == 0) {
			dataTable.getCheckMessage().setSuccess(true);
			dataTable.getCheckMessage().setMessage("PK생성 가능 유무 검사 - 지정한 컬럼(" + primaryColumns + ")으로 PrimaryKey 생성이 가능합니다.");
		} else {
			dataTable.getCheckMessage().setSuccess(false);
			dataTable.getCheckMessage().setMessage("PK생성 가능 유무 검사 - 지정한 컬럼(" + primaryColumns + ")으로 PrimaryKey 생성이 불가능합니다.(중복되는 데이터 수 :" + totalOverlapCount + ")");
		}		
	}
	
	/**
	 * @author 김성원
	 * @since 2024.01.04
	 * 
	 * 커넥션된 데이터베이스의 테이블의 데이터에 대한 데이터 테이블 조회 메서드입니다.
	 */
	public void setDataTableByDBCollectionQuery (ConnectionDB dbCollection, DataTable dataTable) throws Exception {
		if (dataTable.getColumnDatas() == null || dataTable.getColumnDatas().size() == 0) {
			dataTable.getCheckMessage().setMessage("조회할 컬럼정보가 없습니다.");
			dataTable.getCheckMessage().setSuccess(false);
			logger.info("setDataTableByDBConnectionTable : " + dataTable.getCheckMessage().getMessage());
			return;
		}
		
		//DB연결 확인
		CheckMessage message = dbConnectionCheck(dbCollection);
		if (message.isSuccess() == false) {
			dataTable.setCheckMessage(message);
			logger.info("setDataTableByDBConnectionTable DB연결실패 : " + message.getMessage());
			return;
		}
		
		int tableDataTotalRows = 0;
		
		List<LinkedHashMap<String, Object>> tableDataMapList  = new ArrayList<LinkedHashMap<String, Object>>();
		
		//DAO 인스턴스 생성
		DatabaseDAO databaseDAO = createDatabaseDAO(dbCollection);
		
		try {
			tableDataMapList = databaseDAO.getDBCollectionQueryDataList(dataTable);
			dataTable.setRowData(CommonUtil.rowDataMapToList(tableDataMapList));//테이블 데이터 set
			
			//테이블 데이터 총 개수 조회
			tableDataTotalRows = databaseDAO.getDBCollectionQueryDataTotalRows(dataTable);
			dataTable.setTotalRows(tableDataTotalRows);//테이블 데이터 총 개수 set
		}catch(Exception e) {
			
		}finally {
			//DB연결 종료
			databaseDAO.dbConnectionClose();
		}
		
		
		//성공 메세지 입력
		dataTable.getCheckMessage().setSuccess(true);
		dataTable.getCheckMessage().setMessage("(총 조회된 행수 " + tableDataTotalRows + "개 / 현재 조회된 데이터 수 " + tableDataMapList.size() + "개)");
	}
	
	/**
	 * @author 김성원
	 * @since 2024.01.04
	 * 
	 * 데이터 셋 목록 Convert List<TableBasicInfo> to List<ColumnData>
	 */
	/*
	public List<ColumnData> tableColumnToColumnData (List<TableBasicInfo> tableBasicList) throws Exception {
		List<ColumnData> columnDataList = new ArrayList<ColumnData>();
		for (int i = 0; i < tableBasicList.size(); i++) {
			TableBasicInfo tableBasicInfo = tableBasicList.get(i);
			ColumnData columnData = new ColumnData();
		    columnData.setTableName(tableBasicInfo.getTableName());
		    columnData.setColumnNm(tableBasicInfo.getColumnName());
		    columnData.setOrginlColumnNm(tableBasicInfo.getColumnName());
		    columnData.setDisplyColumnNm(tableBasicInfo.getColumnName());
		    columnData.setDbDataType(tableBasicInfo.getDataType());
		    columnData.setDataSize(tableBasicInfo.getSize());
		    columnData.setPkAt(tableBasicInfo.isPrimary());
		    
			columnDataList.add(columnData);
		}
		return columnDataList;
	}
	*/
	
	/**
	 * @author 김성원
	 * @since 2022.01.17
	 * 
	 * 커넥션된 데이터베이스의 실제 테이블 생성
	 */
	public int datasetTableCreate(ConnectionDB dbCollection, DataTable datatable) throws Exception {
			
		int result = 0;
				
		//DAO 인스턴스 생성
		DatabaseDAO databaseDAO = createDatabaseDAO(dbCollection);
		
		try {
			//테이블 컬럼 목록 조회
			result = databaseDAO.datasetTableCreate(datatable);
		}finally {
			//DB연결 종료
			databaseDAO.dbConnectionClose();		
		}
		
		return result;
	}
	
	/**
	 * @author 김성원
	 * @since 2022.01.17
	 * 
	 * 커넥션된 데이터베이스의 실제 테이블 생성
	 */
	public int datasetDefaultTableCreate(ConnectionDB dbCollection, DataTable datatable) throws Exception {
			
		int result = 0;
				
		//DAO 인스턴스 생성		
		DatabaseDAO databaseDAO = createDatabaseDAO(dbCollection);
		
		try {
		//테이블 컬럼 목록 조회
			result = databaseDAO.datasetDefaultTableCreate(datatable);
		}catch(Exception e) {
			
			e.printStackTrace();
			
		}finally {
			//DB연결 종료
			databaseDAO.dbConnectionClose();		
		}
		
		return result;
	}
	
	/**
	 * @author 김성원
	 * @since 2022.01.17
	 * 
	 * 커넥션된 데이터베이스의 실제 테이블 업데이트
	 */
	public void datasetDataUpdate(ConnectionDB dbCollection, DataTable datatable) throws Exception {
						
		//DAO 인스턴스 생성
		DatabaseDAO databaseDAO = createDatabaseDAO(dbCollection);
		//datatable.getColumnDatas().get(0).isAutoIncrement();
		try {
			//테이블 컬럼 목록 조회
			databaseDAO.datasetDataUpdate(datatable);
		}catch(Exception e) {
			e.printStackTrace();
		}
		finally {
			//DB연결 종료
			databaseDAO.dbConnectionClose();		
		}		
	
	}
	
	/**
	 * @author 김성원
	 * @since 2022.01.17
	 * 
	 * 커넥션된 데이터베이스의 실제 테이블 이름 중복검사
	 */
	public int duplicateCheckTableName(ConnectionDB dbCollection, Map info) throws Exception {
	
	
		int result = 0;
		//DAO 인스턴스 생성
		DatabaseDAO databaseDAO = createDatabaseDAO(dbCollection);
		try {
			//테이블 컬럼 목록 조회
			result = databaseDAO.duplicateCheckTableName(info);
		}finally {
			//DB연결 종료
			databaseDAO.dbConnectionClose();
		}	
	
		return result;
	}
	
	/**
	 * @author 김성원
	 * @since 2022.01.17
	 * 
	 * 데이터 셋의 컬럼 정보 목록 조회 SQL에 접근하는 메소드 입니다.(페이징)
	 */
	public List<LinkedHashMap<String, Object>> getRowData (ConnectionDB dbCollection, DataTable dataTable) throws Exception {
				
		List<LinkedHashMap<String, Object>> rowMapData = new ArrayList<LinkedHashMap<String, Object>>();
		
		//DAO 인스턴스 생성
		DatabaseDAO databaseDAO = createDatabaseDAO(dbCollection);
		try {
			//테이블 컬럼 목록 조회
			rowMapData = databaseDAO.getRowData(dataTable);
		}finally {
			//DB연결 종료
			databaseDAO.dbConnectionClose();		
		}	
	
		return rowMapData;
	}
	
	/**
	 * @author 김성원
	 * @since 2022.01.17
	 * 
	 * 데이터 셋의 컬럼 정보 목록 조회 메소드 입니다.(페이징X)
	 */	
	public List<LinkedHashMap<String, Object>> getRowDataAll (ConnectionDB dbCollection, DataTable dataTable) throws Exception {
	
		List<LinkedHashMap<String, Object>> rowMapData = new ArrayList<LinkedHashMap<String, Object>>();
		//DAO 인스턴스 생성
		DatabaseDAO databaseDAO = createDatabaseDAO(dbCollection);
		
		try {
			//테이블 컬럼 목록 조회
			rowMapData = databaseDAO.getRowDataAll(dataTable);
		}finally {
			//DB연결 종료
			databaseDAO.dbConnectionClose();		
		}	
		
		return rowMapData;
	}
	
	/**
	 * @author 김성원
	 * @since 2022.01.17
	 * 
	 * 데이터 셋의 컬럼 정보 목록 총 갯수 조회 메소드 입니다.
	 */
	public int getRowDataTotalRows (ConnectionDB dbCollection, DataTable dataTable) throws Exception {
		
		int result = 0;
		
		//DAO 인스턴스 생성
		DatabaseDAO databaseDAO = createDatabaseDAO(dbCollection);
		
		try {
		//테이블 컬럼 목록 조회
			result = databaseDAO.getRowDataTotalRows(dataTable);		
		}finally {
			//DB연결 종료
			databaseDAO.dbConnectionClose();		
		}		
	
		return result;
	}
	
	/**
	 * @author 김성원
	 * @since 2022.01.17
	 * 
	 * 데이터 셋의 비우기 (데이터 셋 내용삭제)
	 */
	public int emptyDataset (ConnectionDB dbCollection, DataTable dataTable) throws Exception {		
	
		int result = 0;
		//DAO 인스턴스 생성
		DatabaseDAO databaseDAO = createDatabaseDAO(dbCollection);
		
		try {
		//테이블 컬럼 목록 조회
			result = databaseDAO.emptyDataset(dataTable);
		}finally {
			//DB연결 종료
			databaseDAO.dbConnectionClose();		
		}
		
		return result;
	}
	
	/**
	 * @author 김성원
	 * @since 2022.01.17
	 * 
	 *  실제 생성된 컬럼 변경
	 */
	public int tableColumnChange (DatabaseDAO databaseDAO, TableBasicInfo tableBasicInfo) throws Exception {
		
		int result = 0;	
		
		//테이블 컬럼 목록 조회
		result = databaseDAO.tableColumnChange(tableBasicInfo);		
		
		return result;
	}
	
	/**
	 * @author 김성원
	 * @since 2022.01.17
	 * 
	 *  실제 생성된 테이블 키변경
	 */
	public int changePromaryKey (DatabaseDAO databaseDAO, TableBasicInfo tableBasicInfo) throws Exception {
		
		int result = 0;				
		
		//테이블 컬럼 목록 조회
		result = databaseDAO.changePrimaryKey(tableBasicInfo);		
	
		return result;
	}
	
	/**
	 * @author 김성원
	 * @since 2022.01.17
	 * 
	 * 데이터 셋의 ROW 삭제
	 */
	public void datasetDataDelete (ConnectionDB dbCollection, DataTable dataTable) throws Exception {		
	
		//DAO 인스턴스 생성
		DatabaseDAO databaseDAO = createDatabaseDAO(dbCollection);
		
		//테이블 컬럼 목록 조회
	    databaseDAO.datasetDataDelete(dataTable);
		
		//DB연결 종료
		databaseDAO.dbConnectionClose();				
	}
	
	/**
	 * @author 김성원
	 * @since 2022.01.17
	 * 
	 *   AI PK 생성 
	 */
	public int createAutoIncrement (DatabaseDAO databaseDAO, TableBasicInfo tableBasicInfo) throws Exception {
		
		int result = 0;				
		
		//테이블 컬럼 목록 조회
		result = databaseDAO.createAutoIncrement(tableBasicInfo);		
	
		return result;
	}
	
	/**
	 * @author 김성원
	 * @since 2022.01.17
	 * 
	 *   테이블 메타정보 변경 (comments)
	 */
	public void updateTableMetaInfo(ConnectionDB dbCollection,DataTable dataTable) throws Exception {
		
		DatabaseDAO databaseDAO = createDatabaseDAO(dbCollection);
		
		databaseDAO.updateTableMetaInfo(dataTable);
		
	}
	
}
