package com.takensoft.taken_bi_manager.common.util;


import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import jakarta.servlet.annotation.WebListener;
import jakarta.servlet.http.HttpSession;
import jakarta.servlet.http.HttpSessionEvent;
import jakarta.servlet.http.HttpSessionListener;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @author 서영석
 * @since 2023.05.31
 *
 * Custom session 저장소를 만들어 놓고
 * Tomcat이 관리하는 session 생성 또는 소멸시, 발생하는 이벤트를 활용하여 (HttpSessionListener를 통해 이벤트를 받을 수 있음)
 * Custom session 저장소에 추가, 삭제를 통해
 * session을 적절히 controll 하기위한 목적을 가진 Class 입니다.
 */
@WebListener
public class SesssionEventListener implements HttpSessionListener {

    private static final Logger LOGGER = LoggerFactory.getLogger(SesssionEventListener.class);

    //싱글턴패턴 사용을 위한 클래스변수
    public static SesssionEventListener sesssionEventListener = null;

    //싱글턴패턴으로 객체 생성 후 리턴.
    public static synchronized SesssionEventListener getInstance() {
        if(sesssionEventListener == null) {
            sesssionEventListener = new SesssionEventListener();
        }
        return sesssionEventListener;
    }

    /**
     * Custom session 저장소 (tomcat이 관리하고있는 실제 session저장소는 아님) 목록
     * ConcurrentHashMap을 사용하는 이유 :
     * HashMap은 Thread Safe 하지 않기 때문에 싱글턴 패턴에서 해당 session 저장소를 사용하기 위해서는 Thread Safe 한 ConcurrentHashMap를 사용
     * HashTable도 Thread Safe 하지만, 읽기 쓰기 둘다 한개의 Thread만 접근 가능함
     * ConcurrentHashMap은 쓰기에서한 한개의 Tread만 접근 가능하도록 해놓음 (읽기는 여러개의 쓰레드에서 동시 접근 가능)
     */
    private static final Map<String, HttpSession> sessions = new ConcurrentHashMap<>();

    /**
     * @author 서영석
     * @since 2023.05.31
     *
     * @param userId : 사용자(회원) ID
     *
     * userId와 동일한 로그인 session을 가진 session ID 목록 조회
     */
    public synchronized static List<String> duplicationLoginSessionIdSelectListByUserId (String userId) {
        List<String> result = new ArrayList<>();
        try {
            //Custom session 저장소에서 session ID는 다르지만 userId가 같은게 존재하는지 확인
            for (String key : sessions.keySet()) {
                HttpSession session = sessions.get(key);
                if (session == null || session.getAttribute(AuthUtil.LOGIN_USER_SESSION) == null) {
                    //세션이 존재하고, 로그인 정보가 없으면 continue;
                    continue;
                } else {
                    //로그인 정보가 있으면 -> 중복 확인
                    HashMap<String, Object> loginUser = (HashMap<String, Object>) session.getAttribute(AuthUtil.LOGIN_USER_SESSION);
                    String loginUserId = (String) loginUser.get("user_id");

                    /** 사용자(회원) ID와 동일한 로그인 정보 일 시 -> add **/
                    if (userId.equals(loginUserId) == true) {
                        result.add(session.getId());
                    } else {
                        //동일한 로그인 정보가 아니면 -> continue;
                        continue;
                    }
                }
            }
        } catch (NullPointerException e) {
            LOGGER.error(e.toString());
        }

        return result;
    }


    /**
     * @author 서영석
     * @since 2023.05.31
     *
     * 현재 client의 정보와 중복된 로그인 session ID 목록 조회
     *
     * 현재 User의 세션과 Custom session 저장소에 저장된 session 중
     * session ID는 다르지만 userId가 같은 session List return
     */
    public synchronized static List<String> duplicationLoginSessionIdSelectList () {
        List<String> result = new ArrayList<>();

        try {
            //현재 사용자의 session
            HttpSession currentUserSession = CommonUtil.getHttpSession(false);
            //현재 사용자의 session이 존재하고, 로그인 정보가 있을 때
            if (currentUserSession != null && currentUserSession.getAttribute(AuthUtil.LOGIN_USER_SESSION) == null) {
                //현재 사용자의 로그인 정보
                HashMap<String, Object> currentLoginUser = (HashMap<String, Object>) currentUserSession.getAttribute(AuthUtil.LOGIN_USER_SESSION);
                String currentLoginUserId = (String) currentLoginUser.get("user_id");

                //Custom session 저장소에서 session ID는 다르지만 userId가 같은게 존재하는지 확인
                for (String key : sessions.keySet()) {
                    HttpSession session = sessions.get(key);
                    if (session == null || session.getAttribute(AuthUtil.LOGIN_USER_SESSION) == null) {
                        //세션이 존재하고, 로그인 정보가 없으면 continue;
                        continue;
                    } else if (currentUserSession.getId().equals(session.getId()) == true) {
                        //동일한 세션 ID이면 continue;
                        continue;
                    } else {
                        //로그인 정보가 있으면 -> 중복 확인
                        HashMap<String, Object> duplicationLoginUser = (HashMap<String, Object>) session.getAttribute(AuthUtil.LOGIN_USER_SESSION);
                        String duplicationLoginUserId = (String) duplicationLoginUser.get("user_id");

                        /** 동일한 로그인 정보 일 시 -> add **/
                        if (currentLoginUserId.equals(duplicationLoginUserId) == true) {
                            result.add(session.getId());
                        } else {
                            //동일한 로그인 정보가 아니면 -> continue;
                            continue;
                        }
                    }
                }
            }
        } catch (NullPointerException e) {
            LOGGER.error(e.toString());
        }

        return result;
    }

    /**
     * @author 서영석
     * @since 2023.05.31
     *
     * @param userId : 사용자(회원) ID
     *
     * userId와 동일한 로그인 session을 가진 session ID 목록 삭제
     */
    public static int duplicationLoginSessionDeleteByUserId (String userId) {
        int result = 0;
        try {
            List<String> sessionIds = duplicationLoginSessionIdSelectListByUserId(userId);
            for (int i = 0; i < sessionIds.size(); i++) {
                //Custom session 저장소에서 session 무효화 시키기
                sessions.get(sessionIds.get(i)).invalidate();
                //Custom session 저장소에서 session 삭제
                sessions.remove(sessionIds.get(i));
            }
        } catch (NullPointerException e) {
            LOGGER.error(e.toString());
        }
        return result;
    }

    /**
     * @author 서영석
     * @since 2023.05.31
     *
     * 현재 client의 정보와 중복된 로그인 session 전부 무효화 시키기 (완전 삭제는 못시킴)
     */
    public static int duplicationLoginSessionDeleteAll () {
        int result = 0;
        try {
            List<String> sessionIds = duplicationLoginSessionIdSelectList();
            for (int i = 0; i < sessionIds.size(); i++) {
                //Custom session 저장소에서 session 무효화 시키기
                sessions.get(sessionIds.get(i)).invalidate();
                //Custom session 저장소에서 session 삭제
                sessions.remove(sessionIds.get(i));
            }
        } catch (NullPointerException e) {
            LOGGER.error(e.toString());
        }
        return result;
    }

    //난수생성
    public static String generateRandomHex(int length) {
        SecureRandom random = new SecureRandom();
        byte[] randomBytes = new byte[length / 2]; // 16진수 문자열의 길이에 맞게 바이트 배열 크기를 조절

        random.nextBytes(randomBytes);

        // 바이트 배열을 16진수 문자열로 변환
        StringBuilder hexStringBuilder = new StringBuilder();
        for (byte b : randomBytes) {
            hexStringBuilder.append(String.format("%02x", b));
        }

        return hexStringBuilder.toString();
    }

    /**
     * HttpSessionListener Interface로 부터 상속 받은 메소드
     *
     * tomcat에서 session이 생성되었을 때(session timeout), 발생하는 이벤트
     * @param target : session 이벤트(생성) 객체
     *
     * session 생성 이벤트가 발생하면 Custom session 저장소(sessions)에 해당 session ID 값을 Key로 잡고 저장
     * (※ Default로 session에 현재 Client의 IP를 넣어줌)
     */
    @Override
    public void sessionCreated(HttpSessionEvent target) {
        //System.out.println("sessionCreated - 현재 세션 개수 : " + sessions.size() + "개");
        int i = 1;
        for (String key : sessions.keySet()) {
            HttpSession hs = sessions.get(key);
            //System.out.println("sessionCreated - 세션 목록 " + (i++) + ". " + hs.getId());
        }
//        System.out.println("sessionCreated - 이제 추가될 session id : " +  target.getSession().getId());
        //System.out.println("");
        //System.out.println("");

        // session에 key(salt) 값 넣기 시작
        HashMap<String, Object> key = new HashMap<String, Object>();

        // "SHA1PRNG"은 알고리즘 이름
//        SecureRandom random = null;
//        try {
//            random = SecureRandom.getInstance("SHA1PRNG");
//        } catch (NoSuchAlgorithmException e) {
//            e.printStackTrace();
//        }
//        SecureRandom random = new SecureRandom();
//
//        byte[] saltBytes  = new byte[24];
//        byte[] encKeyBytes = new byte[24];
//        byte[] ivBytes = new byte[24];
//        random.nextBytes(saltBytes);
//        random.nextBytes(encKeyBytes);
//        random.nextBytes(ivBytes);
        //SALT 생성
//        String salt = Base64.getEncoder().encodeToString(saltBytes);
//        String ENC_KEY = Base64.getEncoder().encodeToString(encKeyBytes);
//        String iv = Base64.getEncoder().encodeToString(ivBytes);
        String salt = generateRandomHex(32);
        String ENC_KEY = generateRandomHex(32);
        String iv = generateRandomHex(32);
        key.put("salt",salt);
        key.put("ENC_KEY",ENC_KEY);
        key.put("iv",iv);

        target.getSession().setAttribute("key", key);

//        System.out.println("key : "+key);
        //종료


        //현재 Client의 IP set
        target.getSession().setAttribute("ip", CommonUtil.getClientIp());
//        target.getSession().setAttribute("key",);
//        System.out.println("target test1: "+target);
//        System.out.println("target.getSession() test1: "+target.getSession());
//        System.out.println("sessions test1: "+sessions);
//        System.out.println("sessions test2: "+sessions.get(target.getSession().getId()));
//        System.out.println("CommonUtil.getClientIp() test: "+CommonUtil.getClientIp());
//        System.out.println("target.getSession() test: "+target.getSession());

        //세션 저장소에 set
        sessions.put(target.getSession().getId(), target.getSession());
    }

    /**
     * HttpSessionListener Interface로 부터 상속 받은 메소드
     *
     * tomcat에서 session이 소멸되었을 때, 발생하는 이벤트
     * @param target : session 이벤트(소멸) 객체
     *
     * session 제거 이벤트가 발생하면 Custom session 저장소(sessions)에 해당 session ID 값을 가지고 있는 session 무효화 및 제거
     */
    @Override
    public void sessionDestroyed(HttpSessionEvent target) {
        //소멸된 세션 Custom session 저장소에서 가지고 오기
        if (sessions.get(target.getSession().getId()) != null) {
            //소멸된 세션 무효화 시키기 (가능한지 모르겠지만, 일단 작성해봄)
            sessions.get(target.getSession().getId()).invalidate();
            //Custom session 저장소에서 해당 session 삭제
            sessions.remove(target.getSession().getId());
        } else {
            return;
        }
    }
}